@@ -16,9 +16,9 @@ from ConfigParser import ConfigParser
 from copy import deepcopy
 import os.path
 
-from trac.admin import IAdminCommandProvider
+from trac.admin import AdminCommandError, IAdminCommandProvider
 from trac.core import *
-from trac.util import AtomicFile
+from trac.util import AtomicFile, as_bool
 from trac.util.compat import any
 from trac.util.text import printout, to_unicode, CRLF
 from trac.util.translation import _, N_
@@ -27,6 +27,7 @@ __all__ = ['Configuration', 'Option', 'BoolOption', 'IntOption', 'FloatOption',
            'ListOption', 'ChoiceOption', 'PathOption', 'ExtensionOption',
            'OrderedExtensionsOption', 'ConfigurationError']
 
+# Retained for backward-compatibility, use as_bool() instead
 _TRUE_VALUES = ('yes', 'true', 'enabled', 'on', 'aye', '1', 1, True)
 
 _use_default = object()
@@ -391,14 +392,12 @@ class Section(object):
         """Return the value of the specified option as boolean.
         
         This method returns `True` if the option value is one of "yes", "true",
-        "enabled", "on", or "1", ignoring case. Otherwise `False` is returned.
+        "enabled", "on", or non-zero numbers, ignoring case. Otherwise `False`
+        is returned.
 
         Valid default input is a string or a bool. Returns a bool.
         """
-        value = self.get(key, default)
-        if isinstance(value, basestring):
-            value = value.lower() in _TRUE_VALUES
-        return bool(value)
+        return as_bool(self.get(key, default))
 
     def getint(self, key, default=''):
         """Return the value of the specified option as integer.
@@ -697,7 +696,7 @@ class ConfigurationAdmin(Component):
         yield ('config set', '<section> <option> <value>',
                'Set the value for the given option in "trac.ini"',
                self._complete_config, self._do_set)
-    
+
     def _complete_config(self, args):
         if len(args) == 1:
             return self.config.sections()
@@ -705,8 +704,12 @@ class ConfigurationAdmin(Component):
             return [name for (name, value) in self.config[args[0]].options()]
 
     def _do_get(self, section, option):
+        if not self.config.has_option(section, option):
+            raise AdminCommandError(
+                _("Option '%(option)s' doesn't exist in section '%(section)s'",
+                  option=option, section=section))
         printout(self.config.get(section, option))
-        
+
     def _do_set(self, section, option, value):
         self.config.set(section, option, value)
         self.config.save()
@@ -714,6 +717,10 @@ class ConfigurationAdmin(Component):
             self.config.parse_if_needed(force=True) # Full reload
 
     def _do_remove(self, section, option):
+        if not self.config.has_option(section, option):
+            raise AdminCommandError(
+                _("Option '%(option)s' doesn't exist in section '%(section)s'",
+                  option=option, section=section))
         self.config.remove(section, option)
         self.config.save()
         if section == 'inherit' and option == 'file':
