@@ -184,7 +184,9 @@ class SQLiteConnector(Component):
             if os.path.exists(path):
                 raise TracError(_('Database already exists at %(path)s',
                                   path=path))
-            os.makedirs(os.path.split(path)[0])
+            dir = os.path.dirname(path)
+            if not os.path.exists(dir):
+                os.makedirs(dir)
         if isinstance(path, unicode): # needed with 2.4.0
             path = path.encode('utf-8')
         cnx = sqlite.connect(path, timeout=int(params.get('timeout', 10000)))
