@@ -43,8 +43,8 @@ class CachedRepository(Repository):
     def __init__(self, env, repos, log):
         self.env = env
         self.repos = repos
-        self._metadata_id = (self.__class__.__module__ + '.'
-                             + self.__class__.__name__ + '.metadata:'
+        self._metadata_id = (CachedRepository.__module__ + '.'
+                             + CachedRepository.__name__ + '.metadata:'
                              + str(self.repos.id))
         Repository.__init__(self, repos.name, repos.params, log)
 
@@ -146,9 +146,11 @@ class CachedRepository(Repository):
                         != os.path.normcase(self.name):
                     self.log.info("'repository_dir' has changed from %r to %r",
                                   repository_dir, self.name)
-                    raise TracError(_("The 'repository_dir' has changed, a "
-                                      "'trac-admin $ENV repository resync' "
-                                      "operation is needed."))
+                    raise TracError(_("The repository directory has changed, "
+                                      "you should resynchronize the "
+                                      "repository with: trac-admin $ENV "
+                                      "repository resync '%(reponame)s'",
+                                      reponame=self.reponame or '(default)'))
             elif repository_dir is None: # 
                 self.log.info('Storing initial "repository_dir": %s',
                               self.name)
