@@ -26,14 +26,14 @@ from StringIO import StringIO
 
 from genshi.builder import tag
 
-from trac.config import Option, BoolOption, IntOption, _TRUE_VALUES
+from trac.config import Option, BoolOption, IntOption
 from trac.core import *
 from trac.mimeview import Context, Mimeview
 from trac.perm import IPermissionRequestor
 from trac.resource import Resource, ResourceNotFound
 from trac.search import ISearchSource, search_to_sql, shorten_result
 from trac.timeline.api import ITimelineEventProvider
-from trac.util import content_disposition, embedded_numbers, pathjoin
+from trac.util import as_bool, content_disposition, embedded_numbers, pathjoin
 from trac.util.compat import any
 from trac.util.datefmt import from_utimestamp, pretty_timedelta
 from trac.util.text import exception_to_unicode, to_unicode, \
@@ -864,7 +864,7 @@ class ChangesetModule(Component):
                     ('repo-' + repos.reponame,
                      u"\xa0\xa0-\xa0" + (repos.reponame or _('(default)')))
                     for repos in repositories
-                    if repos.params.get('hidden') not in _TRUE_VALUES
+                    if not as_bool(repos.params.get('hidden'))
                     and repos.can_view(req.perm)]
                 filters.sort()
                 add_script(req, 'common/js/timeline_multirepos.js')
@@ -1062,17 +1062,17 @@ class ChangesetModule(Component):
             rev, path = chgset[:sep], chgset[sep:]
         else:
             rev, path = chgset, '/'
-        reponame, repos, path = rm.get_repository_by_path(path)
-        if not reponame:
-            reponame = rm.get_default_repository(formatter.context)
-            if reponame is not None:
-                repos = rm.get_repository(reponame)
-        if path == '/':
-            path = None
+        try:
+            reponame, repos, path = rm.get_repository_by_path(path)
+            if not reponame:
+                reponame = rm.get_default_repository(formatter.context)
+                if reponame is not None:
+                    repos = rm.get_repository(reponame)
+            if path == '/':
+                path = None
 
-        # rendering changeset link
-        if repos:
-            try:
+            # rendering changeset link
+            if repos:
                 changeset = repos.get_changeset(rev)
                 if changeset.can_view(formatter.perm):
                     href = formatter.href.changeset(rev,
@@ -1084,12 +1084,12 @@ class ChangesetModule(Component):
                 errmsg = _("No permission to view changeset %(rev)s "
                            "on %(repos)s", rev=rev,
                            repos=reponame or _('(default)'))
-            except TracError, e:
-                errmsg = to_unicode(e)
-        elif reponame:
-            errmsg = _("Repository '%(repo)s' not found", repo=reponame)
-        else:
-            errmsg = _("No default repository defined")
+            elif reponame:
+                errmsg = _("Repository '%(repo)s' not found", repo=reponame)
+            else:
+                errmsg = _("No default repository defined")
+        except TracError, e:
+            errmsg = to_unicode(e)
         return tag.a(label, class_="missing changeset", title=errmsg)
 
     def _format_diff_link(self, formatter, ns, target, label):
