@@ -47,7 +47,7 @@ from trac.util.text import pretty_size, obfuscate_email_address, \
                            javascript_quote, exception_to_unicode
 from trac.util.datefmt import pretty_timedelta, format_datetime, format_date, \
                               format_time, from_utimestamp, http_date, utc
-from trac.util.translation import _
+from trac.util.translation import _, get_available_locales
 from trac.web.api import IRequestHandler, ITemplateStreamFilter, HTTPNotFound
 from trac.web.href import Href
 from trac.wiki import IWikiSyntaxProvider
@@ -275,7 +275,19 @@ class Chrome(Component):
         memory.""")
 
     htdocs_location = Option('trac', 'htdocs_location', '',
-        """Base URL of the core static resources.""")
+        """Base URL for serving the core static resources below 
+        `/chrome/common/`.
+
+        It can be left empty, and Trac will simply serve those resources
+        itself.
+
+        Advanced users can use this together with
+        [TracAdmin trac-admin ... deploy <deploydir>] to allow serving the
+        static resources for Trac directly from the web server.
+        Note however that this only applies to the `<deploydir>/htdocs/common`
+        directory, the other deployed resources (i.e. those from plugins) 
+        will not be made available this way and additional rewrite 
+        rules will be needed in the web server.""")
 
     metanav_order = ListOption('trac', 'metanav',
                                'login,logout,prefs,help,about', doc=
@@ -289,10 +301,18 @@ class Chrome(Component):
            listed by IDs. See also TracNavigation.""")
 
     logo_link = Option('header_logo', 'link', '',
-        """URL to link to from header logo.""")
+        """URL to link to, from the header logo.""")
 
     logo_src = Option('header_logo', 'src', 'site/your_project_logo.png',
-        """URL of the image to use as header logo.""")
+        """URL of the image to use as header logo.
+        It can be absolute, server relative or relative.
+
+        If relative, it is relative to one of the `/chrome` locations:
+        `site/your-logo.png` if `your-logo.png` is located in the `htdocs`
+        folder within your TracEnvironment;
+        `common/your-logo.png` if `your-logo.png` is located in the
+        folder mapped to the [#trac-section htdocs_location] URL. 
+        Only specifying `your-logo.png` is equivalent to the latter.""")
 
     logo_alt = Option('header_logo', 'alt', 
         "(please configure the [header_logo] section in trac.ini)",
@@ -379,7 +399,11 @@ class Chrome(Component):
         except ImportError:
             babel = None
         if babel is not None:
-            yield 'Babel', get_pkginfo(babel).get('version')
+            info = get_pkginfo(babel).get('version')
+            if not get_available_locales():
+                info += " (translations unavailable)" # No i18n on purpose
+                self.log.warning("Locale data is missing")
+            yield 'Babel', info
 
     # IEnvironmentSetupParticipant methods
 
@@ -865,9 +889,12 @@ class Chrome(Component):
 
     def cc_list(self, cc_field):
         """Split a CC: value in a list of addresses."""
-        if not cc_field:
-            return []
-        return [cc.strip() for cc in cc_field.split(',') if cc]
+        ccs = []
+        for cc in re.split(r'[;,]', cc_field):
+            cc = cc.strip()
+            if cc:
+                ccs.append(cc)
+        return ccs
 
     def format_emails(self, context, value, sep=', '):
         """Normalize a list of e-mails and obfuscate them if needed.
@@ -887,6 +914,15 @@ class Chrome(Component):
                                   email_map and '@' not in author and
                                   email_map.get(author) or author)
 
+    def get_email_map(self):
+        """Get the email addresses of all known users."""
+        email_map = {}
+        if self.show_email_addresses:
+            for username, name, email in self.env.get_known_users():
+                if email:
+                    email_map[username] = email
+        return email_map
+        
     _long_author_re = re.compile(r'.*<([^@]+)@[^@]+>\s*|([^@]+)@[^@]+')
     
     def authorinfo_short(self, author):
